/*
 * GtkMessageWin.C - source file for class GtkMessageWin
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "Model.h"
#include "GtkMessageWin.h"
#include "GtkWriteWin.h"
#include "utils.h"

#include <string.h>
#include <stdio.h>
#include <time.h>

const int BSIZE=1024;

/*
  GtkMessageWin::GtkMessageWin(Model* m) - 
*/
GtkMessageWin::GtkMessageWin(Model* m) : Gtk_Window()
{
    model=m;
    
    set_default_size(600, 800);
    set_title("gtkmail");
    set_policy(TRUE,TRUE,FALSE);
    set_position(GTK_WIN_POS_NONE);
    
    accelGroup = new Gtk_AccelGroup();
    
    itemFactory = new Gtk_ItemFactory_MenuBar("<Main>", *accelGroup);
    itemFactory->create_item("/File", 0, "<Branch>", 0);  
    itemFactory->create_item("/File/Close", "<control>W", "<Item>", ItemFactoryConnector<GtkMessageWin,string>(this,&fileCallback,"Close"));
    itemFactory->create_item("/File/Quit", "<control>Q", "<Item>", ItemFactoryConnector<GtkMessageWin,string>(this,&fileCallback,"Quit"));
    
    messageMenu = (Gtk_Menu*)itemFactory->create_item("/Message", 0, "<Branch>", 0);  
    itemFactory->create_item("/Message/Compose", "<control>M", "<Item>", ItemFactoryConnector<GtkMessageWin,string>(this,&messageCallback,"Compose"));
    
    
    add_accel_group(*accelGroup);
    
    vbox = new Gtk_VBox(FALSE, 0);
    add(*vbox);
    
    vbox->pack_start(itemFactory->get_menubar_widget("<Main>"), FALSE, TRUE, 0);
    
    hbox = new Gtk_HBox(FALSE,0);
    
    messageWin = new Gtk_ScrolledWindow();  
    messageWin->set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);  
    
    messageText = new Gtk_Text();
}
/*
  void addHelpMenu();
*/
void GtkMessageWin::addHelpMenu()
{
    itemFactory->create_item("/Help", 0, "<LastBranch>", 0);  
    itemFactory->create_item("/Help/About", "<control>H", "<Item>", ItemFactoryConnector<GtkMessageWin,string>(this,&helpCallback,"About"));
    
}


/*
  void destroy() - 
*/
void GtkMessageWin::destroy() 
{
    delete_self();
}

void GtkMessageWin::display(char* msg1, char* msg2, double pause)
{
    Gtk_Dialog* dialog;
    Gtk_Label* label1;
    Gtk_Label* label2;
    Gtk_VBox* vbox;
    Gtk_HBox* hbox1;
    Gtk_HBox* hbox2;
    Gtk_Button* okbutton;
    Gtk_Button* cabutton;
    
    char temp1[64], temp2[64];
    
    sprintf(temp1,"%s",msg1);
    if(msg2) {
	sprintf(temp2,"%s",msg2);
    }   
    
    vbox = new Gtk_VBox(FALSE,5);
    
    label1 = new Gtk_Label(temp1);
    label1->show();
    hbox1 = new Gtk_HBox(FALSE,10);
    hbox1->pack_start(*label1,false,true,10);
    hbox1->show();
    if(msg2) {
	label2 = new Gtk_Label(temp2);
	label2->show();
	hbox2 = new Gtk_HBox(FALSE,10);
	hbox2->pack_start(*label2,false,true,10);
	hbox2->show();
    }
    
    dialog = new Gtk_Dialog();
    dialog->set_position(GTK_WIN_POS_CENTER);
    vbox->pack_start(*hbox1,false,true,0);
    if(msg2) {
	vbox->pack_start(*hbox2,false,true,0);
    }
    vbox->show();
    dialog->get_vbox()->pack_start(*vbox,true,true,10);
    
    okbutton = new Gtk_Button("Ok");
    cabutton = new Gtk_Button("Cancel");
    
    dialog->get_action_area()->pack_start(*okbutton,TRUE, TRUE, 25);
    dialog->get_action_area()->pack_start(*cabutton,TRUE, TRUE, 25);
    
    okbutton->show();
    cabutton->show();
    
    if(pause>0) {
	connect_to_method(Gtk_Main::timeout(1000*pause), this, &killWidgetCallback, (Gtk_Widget*)dialog);
    }
    else {
	connect_to_method(okbutton->clicked, this, &killWidget, (Gtk_Widget*)dialog);
	connect_to_method(cabutton->clicked, this, &killWidget, (Gtk_Widget*)dialog);
    }
    
    dialog->show();
}


/*
  void GtkMessageWin::displayAbout() - 
*/
void GtkMessageWin::displayAbout() 
{ 
    display("gtkmail-0.1.1 (c) 1999", "Joe Yandle <yandle@cs.unc.edu>", 0);
}


/*
  void GtkMessageWin::killWidget(Gtk_Widget* data) - 
*/
void GtkMessageWin::killWidget(Gtk_Widget* data)
{
    data->hide();
    connect_to_function(Gtk_Main::idle(), &deleteWidget, data);
}

/*
  int GtkMessageWin::killWidgetCallback(Gtk_Widget* data)
*/
int GtkMessageWin::killWidgetCallback(Gtk_Widget* data)
{
    killWidget(data);
    return 0;
}

/*
  gint GtkMessageWin::delete_event_impl(GdkEventAny *)
*/
gint GtkMessageWin::delete_event_impl(GdkEventAny *)
{
    hide();
    connect_to_function(Gtk_Main::idle(), &deleteWidget, (Gtk_Widget*)this);
    return 1;
}


/*
  void GtkMessageWin::messageComposeCallback() 
*/
void GtkMessageWin::messageComposeCallback() 
{
    GtkWriteWin* writeWin;
    writeWin = new GtkWriteWin(model);
}

void GtkMessageWin::fileCallback(string s)
{
    if(s == "Quit") {
	model->quitModel();
    }
    else if(s == "Close") {
	delete_event_impl(NULL);
    }
}

void GtkMessageWin::messageCallback(string s)
{
    if(s == "Compose") {
	messageComposeCallback();
    }
}


void GtkMessageWin::helpCallback(string s)
{
    if(s == "About") {
	displayAbout();
    }
}

