/*
 * GtkModel.C - source file for class GtkModel
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "View.h"
#include "GtkModel.h"
#include "MysqlDbase.h"
#include "GtkReadWin.h"
#include "GtkWriteWin.h"
#include "GtkProgressWin.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/file.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

//const char* IBOX="/var/spool/mail/jyandle";
const int BSIZE = 1024;

const char *monthIndex1[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
const char *monthIndex2[] = {"January", "Febuary", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
const char *monthIndex3[] = {"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12" };

const long PROGRESS_MIN = 100*1024;

/*
  GtkModel() - constructor
*/
GtkModel::GtkModel(int argc, char** argv) : Gtk_Main(argc,argv)
{
    system("rm -f /tmp/.gtkmail*");
    
    dbase = new MysqlDbase();
    numFolder=0;  

    iboxTime=0;
    readList = new vector<GtkReadWin>();
    writeList = new vector<GtkWriteWin>();
    
    struct utsname myUts;
    
    userName = new char[255];
    inboxFile = new char[255];
    userAddress = new char[255];
    char hostName[255];
    
    uname(&myUts);
    
    gethostname(hostName, 255);
    
    strcpy(userName, getenv("USER"));
    sprintf(inboxFile, "/var/spool/mail/%s", userName);
    sprintf(userAddress, "%s@%s", userName, hostName);
    
    
    if(!dbase->openDbase()) {
	perror("Couldn't open database");
	exit(1);
    }
    
    if(getDbase()->queryFolders() == 0) {
	getDbase()->addFolder("Inbox");
    }
    
    scanFile((char*)inboxFile, "Inbox", true);
    iboxTime=getIboxTime();
    
    connect_to_method(Gtk_Main::timeout(500), this, &GtkModel::timeoutCallback);
}

/*
  ~GtkModel();
*/
GtkModel::~GtkModel()
{
    
}

/*
  virtual Dbase* getDbase();
*/
Dbase* GtkModel::getDbase()
{
    return dbase;
}

/*
  void scanFile(char* fileName, char* folderName, bool delete) - scan file fp for messages, and put into folder folderName
*/
void GtkModel::scanFile(char* fileName, char* folderName, bool deleteFile)
{
    char *lineBuffer, *headerBuffer, *messageBuffer, *headerFrom, *headerTo, *headerSubject, *headerDate, *headerTime, *headerBoundary, *headerContentType, *internalMessage, *viewableBuffer, *attachBuffer;
    bool bunny=true;
    FILE* fp=NULL;
    long headerStart, headerEnd, messageStart, messageEnd;

    struct stat mystat;

    if(stat(fileName,&mystat) == -1) {
	//return;
    }

    int fileSize = mystat.st_size;

    int messageMax=fileSize;
    int attachMax=fileSize;
    int viewableMax=fileSize;
    
    bool useProgWin = (fileSize > PROGRESS_MIN);
    GtkProgressWin* progWin;

    if(useProgWin) {
	progWin = new GtkProgressWin(this);
	char buf[BSIZE];
	sprintf(buf, "Scanning folder %s, please wait.", folderName);
	progWin->setText(buf);
	progWin->setPct(0);
    }

    lineBuffer = new char[BSIZE];
    headerBuffer = new char[64*BSIZE];
    messageBuffer = new char[messageMax];
    viewableBuffer = new char[viewableMax];
    attachBuffer = new char[attachMax];
    
    internalMessage = new char[messageMax];
    internalMessage[0]=0;
    
    headerFrom = new char[BSIZE];
    headerTo = new char[BSIZE];
    headerSubject = new char[BSIZE];
    headerDate = new char[BSIZE];
    headerTime = new char[BSIZE];
    headerBoundary = new char[BSIZE];
    headerContentType = new char[BSIZE];

    int fd = -1;    

    if((fp = fopen(fileName,"r+"))) {
	fd = open(fileName, O_RDONLY);
	if(flock(fd, LOCK_EX) == -1) {
	    fclose(fp);
	    close(fd);
	    return;
	}
	headerStart=ftell(fp);
	while(fgets(lineBuffer,BSIZE,fp) && !isHeaderStart(lineBuffer)) {
	    headerStart=ftell(fp);
	}
	
	while(bunny) {
	    headerBuffer[0]=0;
	    messageBuffer[0]=0;
	    viewableBuffer[0]=0;
	    attachBuffer[0]=0;

	    headerFrom[0]=0;
	    headerTo[0]=0;
	    headerSubject[0]=0;
	    headerDate[0]=0;
	    headerTime[0]=0;
	    headerBoundary[0]=0;
	    headerContentType[0]=0;
	    
	    strcat(headerBuffer, lineBuffer);
	    headerEnd=ftell(fp);
	    while(fgets(lineBuffer,BSIZE,fp) && !isHeaderEnd(lineBuffer)) {
		if(useProgWin) {
		    progWin->setPct(ftell(fp)/double(fileSize));
		    iterate();
		}
		strcat(headerBuffer, lineBuffer);
		if(strstr(lineBuffer, "Date: ")) {
		    strcpy(headerDate, lineBuffer);
		    removeHead(headerDate, "Date: ");
		    parseDateTime(headerDate, headerDate, headerTime);
		    removeNewLine(headerDate);
		    removeNewLine(headerTime);
		}
		else if(strstr(lineBuffer, "From: ")) {
		    strcpy(headerFrom, lineBuffer);
		    removeHead(headerFrom, "From: ");
		    removeNewLine(headerFrom);
		}
		else if(strstr(lineBuffer, "To: ")) {
		    strcpy(headerTo, lineBuffer);
		    removeHead(headerTo, "To: ");	  
		    removeNewLine(headerTo);
		}
		else if(strstr(lineBuffer, "Subject: ")) {
		    strcpy(headerSubject, lineBuffer);
		    removeHead(headerSubject, "Subject: ");
		    removeNewLine(headerSubject);
		}
		else if(isContentType(lineBuffer)) {
		    parseTypeBound(lineBuffer, headerContentType, headerBoundary);
		}
		else if(strstr(lineBuffer, "oundary=") || strstr(lineBuffer, "OUNDARY=")) {
		    char dummy[BSIZE];
		    parseTypeBound(lineBuffer, dummy, headerBoundary);
		}
		headerEnd=ftell(fp);
	    }
	    
	    if(isHeaderEnd(lineBuffer)) {
		
		messageStart=ftell(fp);
		messageEnd=ftell(fp);
		bool isText = false;
		bool isBinary = false;
		bool isAllText = false;
		bool isAttachHeader=false;
		bool isAttachText=false;
		bool haveType=false;
		bool haveEncoding=false;
		bool is7bit=false;
		bool isbase64=false;
		bool isTypeTextPlain=false;
		bool isTypeApplication=false;

		char contentType[BSIZE];
		char contentTransferEncoding[BSIZE];

		if(headerContentType[0] == 0 || isTextPlain(headerContentType)) {
		    isAllText=true;
		}

		while(fgets(lineBuffer,BSIZE,fp) && !isHeaderStart(lineBuffer)) {
		    if(useProgWin) {
			progWin->setPct(ftell(fp)/double(fileSize));
			iterate();
		    }
		    if(isAllText) {
			if(strlen(viewableBuffer) > 0.9*viewableMax) {
			    viewableMax *= 2;
			    char* temp = new char[viewableMax];
			    temp[0]=0;
			    strcpy(temp, viewableBuffer);
			    delete [] viewableBuffer;
			    viewableBuffer = temp;
			}
			strcat(viewableBuffer, lineBuffer);
		    }
		    else {
			if(strstr(lineBuffer, headerBoundary)) {

			    if(strlen(attachBuffer) > 0) {
				dbase->addAttach(folderName, headerSubject, headerFrom, headerTo, headerDate, 
						 headerTime, attachBuffer, contentType, contentTransferEncoding);
			    }
			    isAttachHeader=true;
			    isText=false;
			    isBinary=false;
			    isAttachText=false;
			    haveType=false;
			    haveEncoding=false;
			    is7bit=false;
			    isbase64=false;
			    isTypeTextPlain=false;
			    isTypeApplication=false;
			    contentType[0]=0;
			    contentTransferEncoding[0]=0;

			    attachBuffer[0]=0;
			}
			else if(isAttachHeader) {
			    if(isHeaderEnd(lineBuffer)) {
				isAttachHeader=false;
				isAttachText=true;
				if(!haveType) {
				    isText=true;
				}
				else if(isTypeTextPlain) {
				    if(!haveEncoding || is7bit) {
					isText=true;
				    }
				    else if(isbase64) {
					isBinary=true;
				    }
				    else {
					isBinary=true;
				    }
				}
				else {
				    isBinary=true;
				}

				
			    }
			    
			    else if(isContentType(lineBuffer)) {
				haveType=true;
				if(isTextPlain(lineBuffer) || isMessage(lineBuffer)) {
				    isTypeTextPlain=true;
				}		
				char tempLine[BSIZE];
				strcpy(tempLine, lineBuffer);
				removeHead(tempLine, "Content-Type: ");
				removeNewLine(tempLine);
				strcpy(contentType, tempLine);
			    }
			    else if(isContentEncoding(lineBuffer)) {
				haveEncoding=true;
				if(is7Bit(lineBuffer)) {
				    is7bit=true;
				}
				else if(isBase64(lineBuffer)) {
				    isbase64=true;
				}
				char tempLine[BSIZE];
				strcpy(tempLine, lineBuffer);
				removeHead(tempLine, "Content-Transfer-Encoding: ");
				removeNewLine(tempLine);
				strcpy(contentTransferEncoding, tempLine);
			    }
			}
			else if(isAttachText) {
			    if(isText) {
				if(strlen(viewableBuffer) > 0.9*viewableMax) {
				    viewableMax *= 2;
				    char* temp = new char[viewableMax];
				    temp[0]=0;
				    strcpy(temp, viewableBuffer);
				    delete [] viewableBuffer;
				    viewableBuffer = temp;
				}
				strcat(viewableBuffer, lineBuffer);
			    }
			    else if(isBinary) {
				if(strlen(attachBuffer) > 0.9*attachMax) {
				    attachMax *= 2;
				    char* temp = new char[attachMax];
				    temp[0]=0;
				    strcpy(temp, attachBuffer);
				    delete [] attachBuffer;
				    attachBuffer = temp;
				}
				strcat(attachBuffer, lineBuffer);
			    }
			}
		    }
		    if(strlen(messageBuffer) > 0.9*messageMax) {
			messageMax *= 2;
			char* temp = new char[messageMax];
			temp[0]=0;
			strcpy(temp, messageBuffer);
			delete [] messageBuffer;
			messageBuffer = temp;
		    }
		    strcat(messageBuffer, lineBuffer);
		    messageEnd=ftell(fp);
		}
		
		headerStart=messageEnd;
#ifdef DEBUG
		printf("message date: %s\n", headerDate);
		printf("message time: %s\n", headerTime);
		printf("message from: %s\n", headerFrom);
		printf("message to: %s\n", headerTo);
		printf("message subject: %s\n", headerSubject);
		printf("Headers:\n%s", headerBuffer);
		printf("Message:\n%s", messageBuffer);
#endif
		
		if(isInternal(headerBuffer)) {
		    sprintf(internalMessage, "%s\n%s", headerBuffer, messageBuffer);
		}
		else {
		    dbase->add(folderName, headerSubject, headerFrom, headerTo, headerDate, headerTime, headerBuffer, messageBuffer, viewableBuffer);
		}
		
		if(!isHeaderStart(lineBuffer)) {
		    bunny=false;
		}
	    }
	    else {
		bunny=false;
	    }
	    
	}
	
	if(deleteFile) {
	    fclose(fp);
	    fp = fopen(fileName,"w+");
	    fwrite(internalMessage, sizeof(char), strlen(internalMessage), fp);
	    fclose(fp);
	}
	else {
	    fclose(fp);
	}
	
	if(fd != -1) {
	    flock(fd, LOCK_UN);
	}
    
    }

    if(useProgWin) {
	progWin->hide();
	delete progWin;
    }

/*
    sprintf(copyCommand, "rm %s", tempFileName);
    system(copyCommand);
*/

    delete [] lineBuffer;
    delete [] headerBuffer;
    delete [] messageBuffer;
    delete [] attachBuffer;
    delete [] viewableBuffer;
    delete [] headerFrom;
    delete [] headerTo;
    delete [] headerSubject;
    delete [] headerDate;
    delete [] headerTime;
}

/*
    bool isContentType(char* line);
*/
bool GtkModel::isContentType(char* line)
{
    return (strstr(line, "Content-Type:") ||
	    strstr(line, "Content-type:") ||
	    strstr(line, "CONTENT-Type:") ||
	    strstr(line, "CONTENT-type:") );
}

/*
    bool isTextPlain(char* line);
*/
bool GtkModel::isTextPlain(char* line)
{
    return (strstr(line, "text/plain") ||
	    strstr(line, "text/Plain") ||
	    strstr(line, "text/PLAIN") ||
	    strstr(line, "Text/plain") ||
	    strstr(line, "Text/Plain") ||
	    strstr(line, "Text/PLAIN") ||
	    strstr(line, "TEXT/plain") ||
	    strstr(line, "TEXT/Plain") ||
	    strstr(line, "TEXT/PLAIN") );
}


/*
  bool isInternal(char* headers);
*/
bool GtkModel::isInternal(char* headers)
{
    return ( strstr(headers, "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA") );
}

/*
  void removeNewLine(char* buffer);
*/
void GtkModel::removeNewLine(char* buffer)
{
    int k=strlen(buffer)-1;

    while(buffer[k]=='\n') {
	buffer[k--]=0;
    }
    
/*
  int i=0, j=0;
  char *temp = new char[strlen(buffer)+1];
  strcpy(temp, buffer);
  
  while(temp[i] != 0) {
  if(temp[i] == '\n') {
  i++;
  }
  else {
	    buffer[j++]=temp[i++];
	}
    }
    buffer[j]=0;
*/
}


/*
  void parseDateTime(char* line, char* dateBuffer, char* timeBuffer) - extract date and time from line
*/
void GtkModel::parseDateTime(char* line, char* dateBuffer, char* timeBuffer)
{
    char temp[BSIZE], weekday[BSIZE], year[BSIZE], month[BSIZE], day[BSIZE], zone[BSIZE];
    strcpy(temp, line);
    
    dateBuffer[0]=0;
    timeBuffer[0]=0;
    
    int numConv = 0;
    
    if( (numConv=sscanf(temp, "%s %s %s %s %s %s", weekday, day, month, year, timeBuffer, zone)) < 6) {
	sscanf(temp, "%s %s %s %s %s", day, month, year, timeBuffer, zone);
    }
    
    int i=0;
    while(!strstr(month, monthIndex1[i]) && !strstr(month, monthIndex2[i])) {
	i++;
    }
    strcpy(month, monthIndex3[i]);
    
    sprintf(dateBuffer, "%s-%s-%s", year, month, day);
}

/*
  void parseTypeBound(char* line, char* dateBuffer, char* timeBuffer) - extract date and time from line
*/
void GtkModel::parseTypeBound(char* line, char* typeBuffer, char* boundBuffer)
{
    char temp[BSIZE];
    char* tempPos = NULL;
    strcpy(temp, line);
    
    typeBuffer[0]=0;
    boundBuffer[0]=0;
    
    tempPos = strstr(line, "Boundary=\"");
    if(tempPos == NULL) {
	tempPos = strstr(line, "boundary=");
    }
    if(tempPos == NULL) {
	tempPos = strstr(line, "BOUNDARY=\"");
    }

    if(tempPos == NULL) {
	if(tempPos == NULL) {
	    tempPos = strstr(line, "Boundary=");
	}
	if(tempPos == NULL) {
	    tempPos = strstr(line, "boundary=");
	}
	if(tempPos == NULL) {
	    tempPos = strstr(line, "BOUNDARY=");
	}
	if(tempPos != NULL) {
	    int i=strlen("boundary=");
	    int j=0;
	    while(*(tempPos+i) != '\n' && *(tempPos+i) != ';') {
		boundBuffer[j++] = *(tempPos + i++);
	    }
	    boundBuffer[j]=0;
	}
    }
    else {
	int i=strlen("boundary=\"");
	int j=0;
	while(*(tempPos+i) != '\"' && *(tempPos+i) != ';' && *(tempPos+i) != '\n') {
	    boundBuffer[j++] = *(tempPos + i++);
	}
	boundBuffer[j]=0;
    }

    tempPos = line;
    int i=strlen("Content-Type: ");
    int j=0;
    while(*(tempPos+i) != '\n' && *(tempPos+i) != ';') {
	typeBuffer[j++] = *(tempPos + i++);
    }
    typeBuffer[j]=0;
}

/*
  void removeHead(char* line, char* head);
*/
void GtkModel::removeHead(char* line, char* head)
{
    int headSize=strlen(head);
    int lineSize=strlen(line);
    for(int i=headSize; i<lineSize;i++) {
	line[i-headSize]=line[i];
    }
    line[lineSize-headSize]=0;
}


/*
  bool isHeaderStart(char* line);
*/
bool GtkModel::isHeaderStart(char* line)
{
    char tmp[BSIZE];
    strcpy(tmp, line);
    tmp[5]=0;
    return !strcmp(tmp,"From ");
}

/*
  bool isHeaderEnd(char* line);
*/
bool GtkModel::isHeaderEnd(char* line)
{
    return !strcmp(line,"\n");
}


/*
  time_t getIboxTime() - return time INBOX was last modified
*/
time_t GtkModel::getIboxTime()
{
    struct stat mystat;
    if(stat(inboxFile,&mystat) == -1) {
	return 0;
    }
    else {
	return mystat.st_mtime;
    }
}

/*
  gint timeoutCallBack() - check for new messages and deal with them if there are any
  return TRUE so timeout will be called again
*/
gint GtkModel::timeoutCallback()
{
    if(getIboxTime() != iboxTime) {
	scanFile(inboxFile, "Inbox",true);
	iboxTime=getIboxTime();
	view->updateView();
    }
    return TRUE;
}

/*
  void setView(View* v) - set view field to v
*/
void GtkModel::setView(View* v)
{
    view = v;
}

/*
  void runModel() - dispatch events and deal with them
*/
void GtkModel::runModel()
{
    //view->initFolders(&folderBuffer);
    run();
}

/*
  void quitModel() - end program
*/
void GtkModel::quitModel()
{  
    system("rm -f /tmp/.gtkmail*");
    quit();
}
/*
  virtual char* getUserName();
*/
char* GtkModel::getUserName()
{
    return userName;
}

/*
  virtual char* getUserAddress();
*/
char* GtkModel::getUserAddress()
{
    if(dbase->getMailAddr()) {
	return dbase->getMailAddr();
    }
    else {
	return userAddress;
    }
}

/*
    virtual void iterate();
*/
void GtkModel::iterate()
{
    while(events_pending()) {
	iteration();
    }
}
/*
  bool isContentEncoding(char* line);
*/
bool GtkModel::isContentEncoding(char* line)
{
    return (strstr(line, "Content-Transfer-Encoding:") ||
	    strstr(line, "Content-Transfer-encoding:") ||
	    strstr(line, "Content-Transfer-ENCODING:") ||
	    strstr(line, "Content-transfer-Encoding:") ||
	    strstr(line, "Content-transfer-encoding:") ||
	    strstr(line, "Content-transfer-ENCODING:") ||
	    strstr(line, "Content-TRANSFER-Encoding:") ||
	    strstr(line, "Content-TRANSFER-encoding:") ||
	    strstr(line, "Content-TRANSFER-ENCODING:") ||
	    strstr(line, "content-Transfer-Encoding:") ||
	    strstr(line, "content-Transfer-encoding:") ||
	    strstr(line, "content-Transfer-ENCODING:") ||
	    strstr(line, "content-transfer-Encoding:") ||
	    strstr(line, "content-transfer-encoding:") ||
	    strstr(line, "content-transfer-ENCODING:") ||
	    strstr(line, "content-TRANSFER-Encoding:") ||
	    strstr(line, "content-TRANSFER-encoding:") ||
	    strstr(line, "content-TRANSFER-ENCODING:") ||
	    strstr(line, "CONTENT-Transfer-Encoding:") ||
	    strstr(line, "CONTENT-Transfer-encoding:") ||
	    strstr(line, "CONTENT-Transfer-ENCODING:") ||
	    strstr(line, "CONTENT-transfer-Encoding:") ||
	    strstr(line, "CONTENT-transfer-encoding:") ||
	    strstr(line, "CONTENT-transfer-ENCODING:") ||
	    strstr(line, "CONTENT-TRANSFER-Encoding:") ||
	    strstr(line, "CONTENT-TRANSFER-encoding:") ||
	    strstr(line, "CONTENT-TRANSFER-ENCODING:") );

}

/*
  bool is7Bit(char* line);
*/
bool GtkModel::is7Bit(char* line)
{
    return (strstr(line, "7Bit") ||
	    strstr(line, "7bit") ||
	    strstr(line, "7BIT") );
}

/*
  bool isBase64(char* line);
*/
bool GtkModel::isBase64(char* line)
{
    return (strstr(line, "Base64") ||
	    strstr(line, "base64") ||
	    strstr(line, "BASE64") );
}

/*
  bool isMessage(char* line);  
*/
bool GtkModel::isMessage(char* line)
{
    return (strstr(line, "message") ||
	    strstr(line, "Message") ||
	    strstr(line, "MESSAGE") );
}

