/*
 * GtkModel.h - header file for class GtkModel
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 *
 * This class implements the abstract class Model, using Gtk_Main
 * as one of its base classes.  This allows a seamless integration 
 * with class GtkView.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef GTKMODEL_H
#define GTKMODEL_H

#include "Model.h"
#include "GtkReadWin.h"
#include "GtkWriteWin.h"

#include <gtk--.h>
#include <time.h>
#include <vector.h>

class GtkModel : public Gtk_Main, public Model
{
 public:
    GtkModel(int argc, char** argv);
    ~GtkModel();
    virtual void setView(View* v);
    virtual void runModel();
    virtual void quitModel();
    virtual void scanFile(char* fileName, char* folderName, bool deleteFile);
    virtual Dbase* getDbase();
    virtual char* getUserName();
    virtual char* getUserAddress();
    virtual void iterate();
 private:
    gint timeoutCallback();
    time_t getIboxTime();
    
    bool isHeaderStart(char* line);
    bool isHeaderEnd(char* line);
    bool isInternal(char* headers);
    bool isTextPlain(char* line);
    bool isContentType(char* line);
    bool isContentEncoding(char* line);
    bool is7Bit(char* line);
    bool isBase64(char* line);  
    bool isMessage(char* line);  

    void removeHead(char* line, char* head);
    void parseDateTime(char* line, char* dateBuffer, char* timeBuffer);
    void parseTypeBound(char* line, char* typeBuffer, char* boundBuffer);
    void removeNewLine(char* buffer);
    
    FILE* iboxFile;
    time_t iboxTime;
    gint numFolder;
    vector<GtkReadWin>* readList;
    vector<GtkWriteWin>* writeList;
    
};

#endif
