/*
 * GtkReadWin.C - source file for class GtkReadWin
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "Model.h"
#include "GtkReadWin.h"
#include "GtkWriteWin.h"
#include "utils.h"

#include <string.h>
#include <stdio.h>
#include <time.h>
#include <fcntl.h>

const int BSIZE=1024;

/*
  GtkReadWin::GtkReadWin() - 
*/
GtkReadWin::GtkReadWin(Model* m, char* subj, char* from, char* date, char* time, char* headers, char* viewable) : GtkMessageWin(m)
{
    //mime = new MimeCodec(headers, message, m);
    itemFactory->create_item("/Message/Reply", "<control>R", "<Item>", ItemFactoryConnector<GtkReadWin,string>(this,&messageCallback,"Reply"));
    itemFactory->create_item("/Attach", 0, "<Branch>", 0);
    itemFactory->create_item("/Attach/View Text", "<control>T", "<Item>", ItemFactoryConnector<GtkReadWin,string>(this,&attachCallback,"View Text"));
    itemFactory->create_item("/Attach/View Image", "<control>G", "<Item>", ItemFactoryConnector<GtkReadWin,string>(this,&attachCallback,"View Image"));
    itemFactory->create_item("/Attach/Export", "<control>P", "<Item>", ItemFactoryConnector<GtkReadWin,string>(this,&attachCallback,"Export"));

    addHelpMenu();
    _gtk_string temp0;
    _gtk_string temp1;
    _gtk_string temp2;
    _gtk_string temp3;
    
    subjLabel = new Gtk_Label("Subject: ");
    subjLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    fromLabel = new Gtk_Label("From: ");
    fromLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    dateLabel = new Gtk_Label("Date: ");
    dateLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    timeLabel = new Gtk_Label("Time: ");
    timeLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    attachLabel = new Gtk_Label("Attach: ");
    attachLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    subjEntry = new Gtk_Entry();
    fromEntry = new Gtk_Entry();
    dateEntry = new Gtk_Entry();
    timeEntry = new Gtk_Entry();
    
    subjEntry->set_text(subj);
    fromEntry->set_text(from);
    dateEntry->set_text(date);
    timeEntry->set_text(time);
    
    attachCombo = new Gtk_Combo();
    attachList = NULL;
    
    attachNum=0;
    model->getDbase()->queryAttach(subj, from, date, time);
    while(model->getDbase()->moreRow()) {
	attachNum++;
	char** tempAttach = model->getDbase()->getRow();
	char *tempAttachString = new char[strlen(tempAttach[0]) + strlen(tempAttach[1]) + 16];
	sprintf(tempAttachString, "%d %s %s", attachNum, tempAttach[0], tempAttach[1]);
	attachList = g_list_append(attachList, tempAttachString);
    }
    if(attachList != NULL) {
	attachCombo->set_popdown_strings(attachList);
    }
    
    headerTable = new Gtk_Table(5, 2, FALSE);
    
    headerHBox = new Gtk_HBox(FALSE, 0);
    
    subjLabelHBox = new Gtk_HBox(FALSE, 0);
    fromLabelHBox = new Gtk_HBox(FALSE, 0);
    dateLabelHBox = new Gtk_HBox(FALSE, 0);
    timeLabelHBox = new Gtk_HBox(FALSE, 0);
    attachLabelHBox = new Gtk_HBox(FALSE, 0);
    
    subjEntryHBox = new Gtk_HBox(FALSE, 0);
    fromEntryHBox = new Gtk_HBox(FALSE, 0);
    dateEntryHBox = new Gtk_HBox(FALSE, 0);
    timeEntryHBox = new Gtk_HBox(FALSE, 0);
    attachComboHBox = new Gtk_HBox(FALSE, 0);
    
    subjLabelHBox->pack_start(*subjLabel, FALSE, FALSE, 0);
    fromLabelHBox->pack_start(*fromLabel, FALSE, FALSE, 0);
    dateLabelHBox->pack_start(*dateLabel, FALSE, FALSE, 0);
    timeLabelHBox->pack_start(*timeLabel, FALSE, FALSE, 0);
    attachLabelHBox->pack_start(*attachLabel, FALSE, FALSE, 0);
    
    subjEntryHBox->pack_start(*subjEntry, TRUE, TRUE, 0);
    fromEntryHBox->pack_start(*fromEntry, TRUE, TRUE, 0);
    dateEntryHBox->pack_start(*dateEntry, TRUE, TRUE, 0);
    timeEntryHBox->pack_start(*timeEntry, TRUE, TRUE, 0);
    attachComboHBox->pack_start(*attachCombo, TRUE, TRUE, 0);
    
    headerTable->attach(*dateLabelHBox, 0, 1, 0, 1, 0, 0, 0, 0); 
    headerTable->attach(*timeLabelHBox, 0, 1, 1, 2, 0, 0, 0, 0); 
    headerTable->attach(*fromLabelHBox, 0, 1, 2, 3, 0, 0, 0, 0); 
    headerTable->attach(*subjLabelHBox, 0, 1, 3, 4, 0, 0, 0, 0); 
    headerTable->attach(*attachLabelHBox, 0, 1, 4, 5, 0, 0, 0, 0); 
    
    headerTable->attach(*dateEntryHBox, 1, 2, 0, 1); 
    headerTable->attach(*timeEntryHBox, 1, 2, 1, 2); 
    headerTable->attach(*fromEntryHBox, 1, 2, 2, 3); 
    headerTable->attach(*subjEntryHBox, 1, 2, 3, 4); 
    headerTable->attach(*attachComboHBox, 1, 2, 4, 5); 
    
    headerHBox->pack_start(*headerTable, TRUE, TRUE, 10);
    
    int please = 0;
    
    messageText->set_editable(FALSE);
    ((Gtk_Editable*)messageText)->insert_text(viewable, strlen(viewable), &please);
    
    messageWin->add(messageText);
    
    vbox->pack_start(*headerHBox, FALSE, FALSE, 10);
    vbox->pack_start(*messageWin, TRUE, TRUE, 0);
    
    show_all();
    
}

GtkReadWin::~GtkReadWin()
{
    //printf("Calling destructor\n");
    subjLabel->hide();
    delete subjLabel;
    fromLabel->hide();
    delete fromLabel;
    dateLabel->hide();
    delete dateLabel;
    timeLabel->hide();
    delete timeLabel;
    
    subjEntry->hide();
    delete subjEntry;
    fromEntry->hide();
    delete fromEntry;
    dateEntry->hide();
    delete dateEntry;
    timeEntry->hide();
    delete timeEntry;

    //g_list_foreach(attachList, deleteNodes, NULL);
    for(int i=0;i<attachNum;i++) {
	delete g_list_nth(attachList,i)->data;
    }
    g_list_free(attachList);

    attachCombo->hide();
    delete attachCombo;

    messageText->hide();
    delete messageText;
    messageWin->hide();
    delete messageWin;
    
    delete accelGroup;
    delete itemFactory;
    
    headerTable->hide();
    delete headerTable;
    headerHBox->hide();
    delete headerHBox;
    
    subjLabelHBox->hide();
    delete subjLabelHBox;
    fromLabelHBox->hide();
    delete fromLabelHBox;
    dateLabelHBox->hide();
    delete dateLabelHBox;
    timeLabelHBox->hide();
    delete timeLabelHBox;
    subjEntryHBox->hide();
    delete subjEntryHBox;
    fromEntryHBox->hide();
    delete fromEntryHBox;
    dateEntryHBox->hide();
    delete dateEntryHBox;
    timeEntryHBox->hide();
    delete timeEntryHBox;    
    attachLabelHBox->hide();
    delete attachLabelHBox;
    attachComboHBox->hide();
    delete attachComboHBox;    

    vbox->hide();
    delete vbox;
    hbox->hide();
    delete hbox;
}

void GtkReadWin::messageCallback(string s)
{
    if(s == "Reply") {
	messageReplyCallback();
    }
}

void GtkReadWin::attachCallback(string s)
{
    if(s == "View Text") {
	attachViewTextCallback();
    }
    else if(s == "View Image") {
	attachViewImageCallback();
    }
    else if(s == "Export") {
	attachExportCallback();
    }
}


/*
  void messageReplayCallback();
*/
void GtkReadWin::messageReplyCallback()
{
    GtkWriteWin* temp = new GtkWriteWin(model);
    char replyTo[BSIZE], replySubject[BSIZE];
    strcpy(replyTo,     (char*)(fromEntry->get_text().c_str())); 
    strcpy(replySubject,(char*)(subjEntry->get_text().c_str()));
    
    temp->setTo(replyTo);
    
    if(!strstr(replySubject, "Re:")) {
	sprintf(replySubject, "Re: %s", (char*)(subjEntry->get_text().c_str()));
    }
    temp->setSubject(replySubject);
}


/*
  void attachViewTextCallback();
*/
void GtkReadWin::attachViewTextCallback()
{
    if(attachNum==0) {
	return;
    }
    char readFrom[BSIZE], readSubj[BSIZE], readDate[BSIZE], readTime[BSIZE];
    char* attachText;

    strcpy(readFrom,(char*)(fromEntry->get_text().c_str())); 
    strcpy(readSubj,(char*)(subjEntry->get_text().c_str()));
    strcpy(readDate,(char*)(dateEntry->get_text().c_str())); 
    strcpy(readTime,(char*)(timeEntry->get_text().c_str()));

    /*
      create two pipes, fork and run the attachment through mimencode
     */

    char attachHead[BSIZE];
    int whichAttach;

    strcpy(attachHead, (char*)attachCombo->get_entry()->get_text().c_str());
    sscanf(attachHead, "%d", &whichAttach);
    //printf("You chose attachment number %d\n", whichAttach);
    
    int i=0;
    model->getDbase()->queryAttach(readSubj, readFrom, readDate, readTime);

    char** attachRow;
    while(i++ < whichAttach) {
	attachRow = model->getDbase()->getRow();
    }

    attachText = new char[2*strlen(attachRow[2])];
    strcpy(attachText, attachRow[2]);

    if(is8Bit(attachRow[1]) || is7Bit(attachRow[1])) {
	GtkReadWin* temp;
	temp = new GtkReadWin(model, readSubj, readFrom, readDate, readTime, NULL, attachText );   
    }
    else {
    int spit[2];

    char tempPath[BSIZE];
    sprintf(tempPath, "/tmp/.gtkmail.%d.tmp", rand());

    if(pipe(spit) == -1) {
	perror("Error opening pipe");
	exit(1);
    }

    if(!fork()) {
	dup2(spit[0], 0);
	close(spit[1]);
	if(isBase64(attachRow[1])) {
	    execl(MIMENCODE, "mimencode", "-u", "-o", tempPath, NULL);
	}
	else {
	    execl(MIMENCODE, "mimencode", "-q", "-u", "-o", tempPath, NULL);
	}
    } 
    else {
	close(spit[0]);
	
	int bytesWrote;
	if((bytesWrote = write(spit[1], attachText, strlen(attachText))) == -1) {
	    perror("Error writing to pipe");
	    exit(1);
	}
	else {
	    //printf("%d bytes written out of %d\n", bytesWrote, strlen(attachText));
	}

	close(spit[1]);

	usleep(250000);
	attachText[0]=0;
	char tempBuffer[BSIZE];
	FILE* tempFile = fopen(tempPath, "r");
	int bytesRead=-1;
	while( (bytesRead=fread(tempBuffer, sizeof(char), (BSIZE-1), tempFile)) != 0) {
	    //printf("%d bytes out of %d possible\n", bytesRead, BSIZE-1);
	    tempBuffer[bytesRead]=0;
	    strcat(attachText, tempBuffer);
	}
	GtkReadWin* temp;
	temp = new GtkReadWin(model, readSubj, readFrom, readDate, readTime, NULL, attachText );    	
    }
    }
}

/*
  void attachViewImageCallback();
*/
void GtkReadWin::attachViewImageCallback()
{
    if(attachNum==0) {
	return;
    }
    char readFrom[BSIZE], readSubj[BSIZE], readDate[BSIZE], readTime[BSIZE];
    char* attachText;

    strcpy(readFrom,(char*)(fromEntry->get_text().c_str())); 
    strcpy(readSubj,(char*)(subjEntry->get_text().c_str()));
    strcpy(readDate,(char*)(dateEntry->get_text().c_str())); 
    strcpy(readTime,(char*)(timeEntry->get_text().c_str()));

    /*
      create two pipes, fork and run the attachment through mimencode
     */

    char attachHead[BSIZE];
    int whichAttach;

    strcpy(attachHead, (char*)attachCombo->get_entry()->get_text().c_str());
    sscanf(attachHead, "%d", &whichAttach);
    //printf("You chose attachment number %d\n", whichAttach);
    
    int i=0;
    model->getDbase()->queryAttach(readSubj, readFrom, readDate, readTime);

    char** attachRow;
    while(i++ < whichAttach) {
	attachRow = model->getDbase()->getRow();
    }

    attachText = new char[2*strlen(attachRow[2])];
    strcpy(attachText, attachRow[2]);

    int spit[2];

    char tempPath[BSIZE];
    sprintf(tempPath, "/tmp/.gtkmail.%d.tmp", rand());

    if(pipe(spit) == -1) {
	perror("Error opening pipe");
	exit(1);
    }

    if(!fork()) {
	dup2(spit[0], 0);
	close(spit[1]);
	if(isBase64(attachRow[1])) {
	    execl(MIMENCODE, "mimencode", "-u", "-o", tempPath, NULL);
	}
	else {
	    execl(MIMENCODE, "mimencode", "-q", "-u", "-o", tempPath, NULL);
	}
    } 
    else {
	close(spit[0]);
	
	int bytesWrote;
	if((bytesWrote = write(spit[1], attachText, strlen(attachText))) == -1) {
	    perror("Error writing to pipe");
	    exit(1);
	}
	else {
	    //printf("%d bytes written out of %d\n", bytesWrote, strlen(attachText));
	}

	close(spit[1]);

	if(!fork()) {
	    execlp(IMAGEVIEW, IMAGEVIEW, tempPath, NULL);
	}
    }
}

/*
  void attachExportCallback();
*/
void GtkReadWin::attachExportCallback()
{
    if(attachNum==0) {
	return;
    }
    Gtk_FileSelection* fileSel = new Gtk_FileSelection("Export Attachment");
    fileSel->set_position(GTK_WIN_POS_CENTER);
    connect_to_method(fileSel->get_ok_button()->clicked, this, &attachExportUpdate, fileSel);
    //connect_to_method(fileSel->get_selection_entry()->activate, this, &attachExportUpdate, fileSel);
    connect_to_method(fileSel->get_cancel_button()->clicked, (GtkMessageWin*)this, &GtkMessageWin::killWidget, (Gtk_Widget*)fileSel);
    connect_to_method(fileSel->get_ok_button()->clicked, (GtkMessageWin*)this, &GtkMessageWin::killWidget, (Gtk_Widget*)fileSel);
    //connect_to_method(fileSel->get_selection_entry()->activate, (GtkMessageWin*)this, &GtkMessageWin::killWidget, (Gtk_Widget*)fileSel);
    
    fileSel->show_all();
}

/*
  void attachExportUpdate();
*/
void GtkReadWin::attachExportUpdate(Gtk_FileSelection* fileSel)
{
    char filePath[BSIZE];
    filePath[0]=0;
    
    strcpy(filePath, fileSel->get_filename().c_str());
    
    //killWidget((Gtk_Widget*)fileSel);
    //fileSel->get_cancel_button()->clicked();

    char readFrom[BSIZE], readSubj[BSIZE], readDate[BSIZE], readTime[BSIZE];
    char* attachText;

    strcpy(readFrom,(char*)(fromEntry->get_text().c_str())); 
    strcpy(readSubj,(char*)(subjEntry->get_text().c_str()));
    strcpy(readDate,(char*)(dateEntry->get_text().c_str())); 
    strcpy(readTime,(char*)(timeEntry->get_text().c_str()));

    char attachHead[BSIZE];
    int whichAttach;

    strcpy(attachHead, (char*)attachCombo->get_entry()->get_text().c_str());
    sscanf(attachHead, "%d", &whichAttach);
    //printf("You chose attachment number %d\n", whichAttach);
    
    int i=0;
    model->getDbase()->queryAttach(readSubj, readFrom, readDate, readTime);

    char** attachRow;
    while(i++ < whichAttach) {
	attachRow = model->getDbase()->getRow();
    }

    attachText = new char[2*strlen(attachRow[2])];
    strcpy(attachText, attachRow[2]);

    /*
      create two pipes, fork and run the attachment through mimencode
     */
    int spit[2];

    if(pipe(spit) == -1) {
	perror("Error opening pipe");
	exit(1);
    }

    if(!fork()) {
	dup2(spit[0], 0);
	
	close(spit[1]);
	execl(MIMENCODE, "mimencode", "-u", "-o", filePath, NULL);
    } 
    else {
	close(spit[0]);
	
	int bytesWrote;
	if((bytesWrote = write(spit[1], attachText, strlen(attachText))) == -1) {
	    perror("Error writing to pipe");
	    exit(1);
	}
	else {
	    //printf("%d bytes written out of %d\n", bytesWrote, strlen(attachText));
	}

	close(spit[1]);
    } 
    delete [] attachText;
}
