/*
 * GtkView.C - source file for class GtkView
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "Model.h"
#include "GtkView.h"
#include "GtkReadWin.h"
#include "GtkWriteWin.h"
#include "utils.h"

#include <string.h>
#include <stdio.h>
#include <time.h>

const int BSIZE=1024;

/*
  static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL, 0, "<Branch>" },
  { "/File/Quit",     "<control>Q", gtk_main_quit, 0, NULL },
  { "/_Options",      NULL,         NULL, 0, "<Branch>" },
  { "/Options/Test",  NULL,         NULL, 0, NULL },
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About",   NULL,         NULL, 0, NULL },
  };
*/

GtkView::GtkView(Model* m) : Gtk_Window()
{
    setModel(m);

    //folderHBox = new Gtk_HBox(FALSE, 10);
    //folderLabel = new Gtk_Label("Folder");
    //folderCombo = new Gtk_Combo();
    //folderCombo->get_entry()->set_editable(false);
    //folderCombo->set_use_arrows_always(true);
    folderGList=NULL;
    //folderHBox->pack_start(*folderLabel, FALSE, FALSE, 10);
    //folderHBox->pack_start(*folderCombo, TRUE, TRUE, 10);
    
    numRows=0;
    numFolder=0;
    folderSelect=-1;
    messageSelect=-1;
    
    set_default_size(680, 250);
    set_title("gtkmail");
    set_policy(TRUE,TRUE,FALSE);
    set_position(GTK_WIN_POS_CENTER);
    
    accelGroup = new Gtk_AccelGroup();
    
    itemFactory = new Gtk_ItemFactory_MenuBar("<Main>", *accelGroup);
    itemFactory->create_item("/File", 0, "<Branch>", 0);  
    itemFactory->create_item("/File/Quit", "<control>Q", "<Item>", ItemFactoryConnector<GtkView,string>(this,&fileCallback,"Quit"));
    
    itemFactory->create_item("/Folder", 0, "<Branch>", 0);  
    //itemFactory->create_item("/Folder/View", "<control>L", "<Item>", ItemFactoryConnector<GtkView,string>(this,&folderCallback,"View"));
    itemFactory->create_item("/Folder/New", "<control>N", "<Item>", ItemFactoryConnector<GtkView,string>(this,&folderCallback,"New"));
    itemFactory->create_item("/Folder/Import", "<control>T", "<Item>", ItemFactoryConnector<GtkView,string>(this,&folderCallback,"Import"));
    itemFactory->create_item("/Folder/Delete", "<control>E", "<Item>", ItemFactoryConnector<GtkView,string>(this,&folderCallback,"Delete"));
    itemFactory->create_item("/Folder/Rename", "<control>A", "<Item>", ItemFactoryConnector<GtkView,string>(this,&folderCallback,"Rename"));
    
    messageMenu = (Gtk_Menu*)itemFactory->create_item("/Message", 0, "<Branch>", 0);  
    itemFactory->create_item("/Message/View", "<control>V", "<Item>", ItemFactoryConnector<GtkView,string>(this,&messageCallback,"View"));
    itemFactory->create_item("/Message/Reply", "<control>R", "<Item>", ItemFactoryConnector<GtkView,string>(this,&messageCallback,"Reply"));
    itemFactory->create_item("/Message/Compose", "<control>M", "<Item>", ItemFactoryConnector<GtkView,string>(this,&messageCallback,"Compose"));

    itemFactory->create_item("/Message/Move To", 0, "<Branch>", 0);  

    itemFactory->create_item("/Message/Delete", "<control>D", "<Item>", ItemFactoryConnector<GtkView,string>(this,&messageCallback,"Delete"));
    
    itemFactory->create_item("/Options", 0, "<Branch>", 0);  
    itemFactory->create_item("/Options/Change Address", "<control>G", "<Item>", ItemFactoryConnector<GtkView,string>(this,&optionsCallback,"Change Address"));
    
    itemFactory->create_item("/Help", 0, "<LastBranch>", 0);  
    itemFactory->create_item("/Help/About", "<control>H", "<Item>", ItemFactoryConnector<GtkView,string>(this,&helpCallback,"About"));
    
    add_accel_group(*accelGroup);
    
    vbox = new Gtk_VBox(FALSE, 0);
    add(*vbox);
    
    vbox->pack_start(itemFactory->get_menubar_widget("<Main>"), FALSE, TRUE, 0);
    //vbox->pack_start(*folderHBox, FALSE, TRUE, 0);
    
    hbox = new Gtk_HBox(FALSE,0);
    
    
    char* folderTitles[] = {"Folder"};
    
    folderList = new Gtk_CList(1,(const gchar**)folderTitles);
    folderList->set_column_justification(0,GTK_JUSTIFY_LEFT);
    folderList->set_column_width(0,75);
    folderList->set_selection_mode(GTK_SELECTION_SINGLE );
    folderList->set_usize(120,-1);
    folderList->set_sort_column(0);
    folderList->set_sort_type(GTK_SORT_ASCENDING);
    
    folderListWin = new Gtk_ScrolledWindow();
    folderListWin->set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);  
    
    //folderListWin->set_usize(100, -1);  
    folderListWin->add(folderList);
    folderListWin->show();
    hbox->pack_start(*folderListWin, FALSE, FALSE, 0);
    
    initFolders();
    
    char* messageTitles[] = {"Date","Time", "From","Subject"};
    
    messageList = new Gtk_CList(4,(const gchar**)messageTitles);
    for(int i=0;i<3;i++) {
	
    }
    messageList->set_column_justification(0,GTK_JUSTIFY_CENTER);
    messageList->set_column_justification(1,GTK_JUSTIFY_CENTER);
    messageList->set_column_justification(2,GTK_JUSTIFY_LEFT);
    messageList->set_column_justification(3,GTK_JUSTIFY_LEFT);
    messageList->set_column_width(0,90);
    messageList->set_column_width(1,60);
    messageList->set_column_width(2,200);
    messageList->set_column_width(3,150);
    messageList->set_selection_mode(GTK_SELECTION_SINGLE );
    
    messageListWin = new Gtk_ScrolledWindow();
    
    messageListWin->set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);  
    messageListWin->add(messageList);
    hbox->pack_start(*messageListWin, TRUE, TRUE, 0);
    
    vbox->pack_start(*hbox, TRUE, TRUE, 0);
    
    connect_to_method(messageList->select_row, this, &messageSMade);
    connect_to_method(messageList->unselect_row, this, &messageSLost);
    
    connect_to_method(folderList->select_row, this, &folderSMade);
    connect_to_method(folderList->unselect_row, this, &folderSLost);

    //connect_to_method(folderCombo->get_entry()->activate, this, &folderComboActive);
    //connect_to_method(folderCombo->get_list()->selection_changed, this, &folderComboActive);
    
    show_all();
}

/*
  void folderComboActive();
*/
void GtkView::folderComboActive()
{
    printf("We got folderComboActive\n");
}


/*
  void initFolders() - initialize folders
*/
void GtkView::initFolders()
{
    char** tempRow;
    char* tempName;
    char tempPath[BSIZE];
    char tempAccl[BSIZE];
    bool bunny = true;
    
    while(bunny) {
	if(model->getDbase()->queryFolders()) {
	    bunny = false;
	}
    }
    
    folderList->freeze();
    while(numFolder > 0) {
	folderList->remove_row(numFolder-1);
	numFolder--;
    }

    for(unsigned int i=0; i<g_list_length(folderGList); i++) {
	char tempTemp[BSIZE];
	sprintf(tempTemp, "/Message/Move To/%s", (char*)g_list_nth(folderGList, i)->data);
	itemFactory->delete_item(tempTemp);
	delete g_list_nth(folderGList,i)->data;
    }
    g_list_free(folderGList);
    folderGList = NULL;
    
    int j=0;
    while(model->getDbase()->moreRow()) {
	tempName = new char[255];
	tempRow = model->getDbase()->getRow();
	strcpy(tempName, tempRow[0]);
	folderList->append((const gchar**)(&tempName));
	folderGList = g_list_append(folderGList, tempName);
	numFolder++;

	sprintf(tempPath, "/Message/Move To/%s", tempName);
	
	if(j < 10) {
	    sprintf(tempAccl, "<control>%d", j++);
	}
	else {
	    strcpy(tempAccl, "");
	}
	itemFactory->create_item(tempPath, tempAccl, "<Item>", 
				 ItemFactoryConnector<GtkView,string>(this,&messageMoveToCallback,tempName));
    }

    //folderCombo->set_popdown_strings(folderGList);
    
    //folderList->sort();
    folderList->thaw();
    show_all();
}


/*
  void destroy() - 
*/
void GtkView::destroy() 
{
    model->quitModel();
}

void GtkView::display(char* msg1, char* msg2, double pause)
{
    Gtk_Dialog* dialog;
    Gtk_Label* label1;
    Gtk_Label* label2;
    Gtk_VBox* vbox;
    Gtk_HBox* hbox1;
    Gtk_HBox* hbox2;
    Gtk_Button* okbutton;
    Gtk_Button* cabutton;
    
    char temp1[64], temp2[64];
    
    sprintf(temp1,"%s",msg1);
    if(msg2) {
	sprintf(temp2,"%s",msg2);
    }   
    
    vbox = new Gtk_VBox(FALSE,5);
    
    label1 = new Gtk_Label(temp1);
    label1->show();
    hbox1 = new Gtk_HBox(FALSE,10);
    hbox1->pack_start(*label1,false,true,10);
    hbox1->show();
    if(msg2) {
	label2 = new Gtk_Label(temp2);
	label2->show();
	hbox2 = new Gtk_HBox(FALSE,10);
	hbox2->pack_start(*label2,false,true,10);
	hbox2->show();
    }
    
    dialog = new Gtk_Dialog();
    dialog->set_position(GTK_WIN_POS_CENTER);
    vbox->pack_start(*hbox1,false,true,0);
    if(msg2) {
	vbox->pack_start(*hbox2,false,true,0);
    }
    vbox->show();
    dialog->get_vbox()->pack_start(*vbox,true,true,10);
    
    okbutton = new Gtk_Button("Ok");
    cabutton = new Gtk_Button("Cancel");
    
    dialog->get_action_area()->pack_start(*okbutton,TRUE, TRUE, 25);
    dialog->get_action_area()->pack_start(*cabutton,TRUE, TRUE, 25);
    
    okbutton->show();
    cabutton->show();
    
    if(pause>0) {
	connect_to_method(Gtk_Main::timeout(1000*pause), this, &killWidgetCallback, (Gtk_Widget*)dialog);
    }
    else {
	connect_to_method(okbutton->clicked, this, &killWidget, (Gtk_Widget*)dialog);
	connect_to_method(cabutton->clicked, this, &killWidget, (Gtk_Widget*)dialog);
    }
    
    dialog->show();
}



void GtkView::displayAbout() 
{ 
    display("gtkmail-0.1.1 (c) 1999", "Joe Yandle <yandle@cs.unc.edu>", 0);
}

/*
  void GtkView::messageComposeCallback() 
*/
void GtkView::messageComposeCallback() 
{
    GtkWriteWin* writeWin;
    writeWin = new GtkWriteWin(model);
}

/*
  void messageDeleteCallback();
*/
void GtkView::messageDeleteCallback()
{
    _gtk_string messageSubj;
    _gtk_string messageFrom;
    _gtk_string messageDate;
    _gtk_string messageTime;
    
    messageList->get_text(messageSelect, 3, &messageSubj);
    messageList->get_text(messageSelect, 2, &messageFrom);
    messageList->get_text(messageSelect, 0, &messageDate);
    messageList->get_text(messageSelect, 1, &messageTime);
    
    model->getDbase()->deleteMessage((char*)(messageSubj.c_str()), (char*)(messageFrom.c_str()), (char*)(messageDate.c_str()), (char*)(messageTime.c_str()));
    
    updateView();
}


void GtkView::updateView()
{
    if(folderSelect != -1) {
	int temp = folderSelect;
	folderSLost(temp, 0, NULL);
	folderSMade(temp,0,NULL);
    }
}

void GtkView::resetList()
{
    
}


void GtkView::messageSMade(gint row, gint column, GdkEvent* button)
{
    messageSelect = row;
    if(button && button->type == GDK_2BUTTON_PRESS) {
	messageViewCallback();
    }
}


void GtkView::messageSLost(gint row, gint column, GdkEvent* button)
{
    messageSelect = -1;
}


void GtkView::folderSMade(gint row, gint column, GdkEvent* button)
{
    folderSelect = row;

    if(button && (button->type == GDK_2BUTTON_PRESS || button->type == GDK_3BUTTON_PRESS)) {
	return;
    }

    _gtk_string temp;
    char** ftemp;
    
    messageList->freeze();
    folderList->get_text(row,column,&temp);
    if(model->getDbase()->queryHead("folder",(char*)(temp.c_str()))) {
	while(model->getDbase()->moreRow()) {
	    ftemp=model->getDbase()->getRow();
	    messageList->append((const gchar**)ftemp);
	    numRows++;
	}
    }
    messageList->thaw();
}


void GtkView::folderSLost(gint row, gint column, GdkEvent* button)
{
    messageList->freeze();
    folderSelect = -1;
    for(int i=0;i<numRows;i++) {
	messageList->remove_row(0);
    }
    numRows=0;
    messageList->thaw();
}


void GtkView::handleEvents()
{
    while (gtk_events_pending())
	gtk_main_iteration();
}


void GtkView::killWidget(Gtk_Widget* data)
{
    data->hide();
    connect_to_function(Gtk_Main::idle(), &deleteWidget, data);
}


int GtkView::killWidgetCallback(Gtk_Widget* data)
{
    killWidget(data);
    return 0;
}


void GtkView::setModel(Model* m)
{
    model = m;
}


gint GtkView::delete_event_impl(GdkEventAny *)
{
    model->quitModel();
    return 0;
}

/*
  void messageMoveToCallback()
*/
void GtkView::messageMoveToCallback(string menuItem)
{
    if(messageSelect == -1) {
	return;
    }
    
    _gtk_string messageSubj;
    _gtk_string messageFrom;
    _gtk_string messageDate;
    _gtk_string messageTime;
    
    messageList->get_text(messageSelect, 3, &messageSubj);
    messageList->get_text(messageSelect, 2, &messageFrom);
    messageList->get_text(messageSelect, 0, &messageDate);
    messageList->get_text(messageSelect, 1, &messageTime);
    
    model->getDbase()->changeFolder((char*)(menuItem.c_str()),  (char*)(messageSubj.c_str()), (char*)(messageFrom.c_str()), (char*)(messageDate.c_str()), (char*)(messageTime.c_str()));
    updateView();
}

/*
  void folderNewCallback();
*/
void GtkView::folderNewCallback()
{
    Gtk_Dialog* aboutd;
    Gtk_Button* okbutton;
    Gtk_Button* cabutton;
    Gtk_Entry* name_entry;
    Gtk_HBox* hbox;
    
    hbox = new Gtk_HBox(TRUE,0);
    
    okbutton = new Gtk_Button("Ok");
    cabutton = new Gtk_Button("Cancel");
    aboutd = new Gtk_Dialog();
    
    char dtitle[256];
    sprintf(dtitle, "New Folder");
    aboutd->set_title(dtitle);
    
    
    name_entry = new Gtk_Entry();
    //name_entry->set_text("");
    
    hbox->pack_start(*name_entry, FALSE, TRUE, 20);
    
    aboutd->get_vbox()->pack_start(*hbox, TRUE, TRUE, 10);
    aboutd->get_action_area()->pack_start(*okbutton,TRUE, TRUE, 25);
    aboutd->get_action_area()->pack_start(*cabutton,TRUE, TRUE, 25);
    aboutd->set_position(GTK_WIN_POS_CENTER);
    
    okbutton->show();
    cabutton->show();
    name_entry->show();
    hbox->show();
    aboutd->show();
    
    connect_to_method(name_entry->activate,this,  &folderNewUpdate, (Gtk_Entry*)name_entry);
    connect_to_method(okbutton->clicked,this, &folderNewUpdate, (Gtk_Entry*)name_entry);
    connect_to_method(name_entry->activate, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(okbutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(cabutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);  
}

/*
  void folderImportCallback();
*/
void GtkView::folderImportCallback()
{
    Gtk_FileSelection* fileSel = new Gtk_FileSelection("Import Folder");
    fileSel->set_position(GTK_WIN_POS_CENTER);
    connect_to_method(fileSel->get_ok_button()->clicked, this, &folderImportUpdate, fileSel);
    //connect_to_method(fileSel->get_selection_entry()->activate, this, &folderImportUpdate, fileSel);
    //connect_to_method(fileSel->get_ok_button()->clicked, this, &killWidget, (Gtk_Widget*)fileSel);
    //connect_to_method(fileSel->get_selection_entry()->activate, this, &killWidget, (Gtk_Widget*)fileSel);
    connect_to_method(fileSel->get_cancel_button()->clicked, this, &killWidget, (Gtk_Widget*)fileSel);
    
    fileSel->show_all();
}

/*
  void folderDeleteCallback();
*/
void GtkView::folderDeleteCallback()
{
    if(folderSelect == -1) {
	return;
    }
    
    _gtk_string name;
    folderList->get_text(folderSelect,0,&name);
    
    model->getDbase()->deleteFolder((char*)(name.c_str()));
    initFolders();
    updateView();
    
}

/*
  void folderRenameCallback();
*/
void GtkView::folderRenameCallback()
{
    if(folderSelect == -1) {
	return;
    }
    
    Gtk_Dialog* aboutd;
    Gtk_Button* okbutton;
    Gtk_Button* cabutton;
    Gtk_Entry* name_entry;
    Gtk_HBox* hbox;
    
    hbox = new Gtk_HBox(TRUE,0);
    
    okbutton = new Gtk_Button("Ok");
    cabutton = new Gtk_Button("Cancel");
    aboutd = new Gtk_Dialog();
    
    _gtk_string name;
    folderList->get_text(folderSelect,0,&name);
    
    char dtitle[256];
    sprintf(dtitle, "Edit name for %s", name.c_str());
    aboutd->set_title(dtitle);
    
    
    name_entry = new Gtk_Entry();
    name_entry->set_text(name.c_str());
    
    hbox->pack_start(*name_entry, FALSE, TRUE, 20);
    
    aboutd->get_vbox()->pack_start(*hbox, TRUE, TRUE, 10);
    aboutd->get_action_area()->pack_start(*okbutton,TRUE, TRUE, 25);
    aboutd->get_action_area()->pack_start(*cabutton,TRUE, TRUE, 25);
    aboutd->set_position(GTK_WIN_POS_CENTER);
    
    okbutton->show();
    cabutton->show();
    name_entry->show();
    hbox->show();
    aboutd->show();
    
    connect_to_method(name_entry->activate,this,  &folderRenameUpdate, name_entry);
    connect_to_method(okbutton->clicked,this, &folderRenameUpdate, name_entry);
    connect_to_method(name_entry->activate, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(okbutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(cabutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);  
    
}


/*
  void GtkView::folderNewUpdate()
*/
void GtkView::folderNewUpdate(Gtk_Entry* entry)
{
    _gtk_string newName;
    
    newName = entry->get_text();
    
    model->getDbase()->addFolder((char*)(newName.c_str()));
    initFolders();
    updateView();
}

/*
  void GtkView::folderImportUpdate()
*/
void GtkView::folderImportUpdate(Gtk_FileSelection* fileSel)
{
    char filePath[BSIZE], fileName[BSIZE];
    
    strcpy(filePath, fileSel->get_filename().c_str());
    strcpy(fileName, fileSel->get_selection_entry()->get_text().c_str());
    
    killWidget((Gtk_Widget*)fileSel);
    
    model->scanFile(filePath, fileName, false);
    model->getDbase()->addFolder(fileName);
    initFolders();
    updateView();
}

/*
  void GtkView::folderDeleteUpdate()
*/
void GtkView::folderDeleteUpdate()
{
    
}

/*
  void GtkView::folderRenameUpdate()
*/
void GtkView::folderRenameUpdate(Gtk_Entry* entry)
{
    _gtk_string oldName, newName;
    
    folderList->get_text(folderSelect,0,&oldName);
    newName = entry->get_text();
    
    model->getDbase()->renameFolder((char*)(oldName.c_str()), (char*)(newName.c_str()));
    initFolders();
    updateView();
}

/*
  void optionsChangeAddressCallback();
*/
void GtkView::optionsChangeAddressCallback()
{
    Gtk_Dialog* aboutd;
    Gtk_Button* okbutton;
    Gtk_Button* cabutton;
    Gtk_Entry* name_entry;
    Gtk_HBox* hbox;
    
    hbox = new Gtk_HBox(TRUE,0);
    
    okbutton = new Gtk_Button("Ok");
    cabutton = new Gtk_Button("Cancel");
    aboutd = new Gtk_Dialog();
    
    char dtitle[256];
    sprintf(dtitle, "Change Mail Address");
    aboutd->set_title(dtitle);
    
    
    name_entry = new Gtk_Entry();
    
    name_entry->set_text(model->getUserAddress());
    
    hbox->pack_start(*name_entry, FALSE, TRUE, 20);
    
    aboutd->get_vbox()->pack_start(*hbox, TRUE, TRUE, 10);
    aboutd->get_action_area()->pack_start(*okbutton,TRUE, TRUE, 25);
    aboutd->get_action_area()->pack_start(*cabutton,TRUE, TRUE, 25);
    aboutd->set_position(GTK_WIN_POS_CENTER);
    
    okbutton->show();
    cabutton->show();
    name_entry->show();
    hbox->show();
    aboutd->show();
    
    connect_to_method(name_entry->activate,this,  &optionsChangeAddressUpdate, (Gtk_Entry*)name_entry);
    connect_to_method(okbutton->clicked,this, &optionsChangeAddressUpdate, (Gtk_Entry*)name_entry);
    connect_to_method(name_entry->activate, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(okbutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);
    connect_to_method(cabutton->clicked, this, &killWidget, (Gtk_Widget*)aboutd);    
}


/*  
    void optionsChangeAddressUpdate(Gtk_Entry* entry);
*/
void GtkView::optionsChangeAddressUpdate(Gtk_Entry* entry)
{
    _gtk_string newName;
    
    newName = entry->get_text();
    
    model->getDbase()->setMailAddr((char*)(newName.c_str()));
}

void GtkView::fileCallback(string s)
{
    if(s == "Quit") {
	model->quitModel();
    }
}

void GtkView::folderCallback(string s)
{
    if(s == "New") {
	folderNewCallback();
    }
    else if(s == "Import") {
	folderImportCallback();
    }
    else if(s == "Delete") {
	folderDeleteCallback();
    }
    else if(s == "Rename") {
	folderRenameCallback();
    }
}

void GtkView::messageCallback(string s)
{
    if(s == "Compose") {
	messageComposeCallback();
    }
    else if(s == "Delete") {
	messageDeleteCallback();
    }
    else if(s == "View") {
	messageViewCallback();
    }
    
    else if(s == "Reply") {
	messageReplyCallback();
    }
    
    
}

void GtkView::optionsCallback(string s)
{
    if(s == "Change Address") {
	optionsChangeAddressCallback();
    }
}

void GtkView::helpCallback(string s)
{
    if(s == "About") {
	displayAbout();
    }
}

/*
  void messageViewCallback();
*/
void GtkView::messageViewCallback()
{
    if(messageSelect == -1) {
	return;
    }

    GtkReadWin* messageWin;
    
    _gtk_string messageSubj;
    _gtk_string messageFrom;
    _gtk_string messageDate;
    _gtk_string messageTime;
    
    messageList->get_text(messageSelect, 3, &messageSubj);
    messageList->get_text(messageSelect, 2, &messageFrom);
    messageList->get_text(messageSelect, 0, &messageDate);
    messageList->get_text(messageSelect, 1, &messageTime);
    
    char *messageHeaders, *messageViewable;
    
    messageHeaders = new char[64*BSIZE];
    messageViewable = new char[BSIZE*BSIZE];
    
    if(model->getDbase()->queryMessage((char*)(messageSubj.c_str()), (char*)(messageFrom.c_str()), (char*)(messageDate.c_str()), (char*)(messageTime.c_str()))) {
	char** temp = model->getDbase()->getRow();
	strcpy(messageHeaders, temp[0]);
	strcpy(messageViewable, temp[2]);
	
	messageWin = new GtkReadWin(model, 
				    (char*)(messageSubj.c_str()), 
				    (char*)(messageFrom.c_str()), 
				    (char*)(messageDate.c_str()), 
				    (char*)(messageTime.c_str()), 
				    messageHeaders, messageViewable);
	
    }
    delete [] messageHeaders;
    delete [] messageViewable;    
}

/*
  void messageReplyCallback();
*/
void GtkView::messageReplyCallback()
{
    if(messageSelect == -1) {
	return;
    }

    GtkWriteWin* temp = new GtkWriteWin(model);
    char replyTo[BSIZE], replySubject[BSIZE];
    _gtk_string messageSubj;
    _gtk_string messageFrom;
    
    messageList->get_text(messageSelect, 3, &messageSubj);
    messageList->get_text(messageSelect, 2, &messageFrom);

    strcpy(replyTo,     (char*)(messageFrom.c_str())); 
    strcpy(replySubject,(char*)(messageSubj.c_str()));
    
    temp->setTo(replyTo);
    if(!strstr(replySubject, "Re:")) {
	sprintf(replySubject, "Re: %s", (char*)(messageSubj.c_str()));
    }
    temp->setSubject(replySubject);
}
