/*
 * GtkView.h - header file for class GtkView
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 *
 * GtkView provides an implementation of the abstract class View, using
 * Gtk_Window as one of its base classes.  It uses the Gtk-- event handling
 * model (the connect_to_* functions)..
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef GTKVIEW_H
#define GTKVIEW_H

#include "View.h"
#include <gtk--.h>

class GtkView : public Gtk_Window, public View
{
 public:
    GtkView(Model* m);
    virtual void setModel(Model* m);
    virtual void updateView();
    virtual void handleEvents();
    virtual void display(char* msg1, char* msg2, double pause);
    virtual void initFolders();
    gint delete_event_impl(GdkEventAny *);
 private:
    
    void fileCallback(string s);
    void folderCallback(string s);
    void messageCallback(string s);
    void optionsCallback(string s);
    void helpCallback(string s);
    
    void destroy();
    void killWidget(Gtk_Widget* data);
    int killWidgetCallback(Gtk_Widget* data);
    void displayAbout();
    //void editNameCallback();
    void folderSMade(gint row, gint column, GdkEvent* button);
    void folderSLost(gint row, gint column, GdkEvent* button);
    void messageSMade(gint row, gint column, GdkEvent* button);
    void messageSLost(gint row, gint column, GdkEvent* button);
    //void updateNameCallback(Gtk_Entry* data);
    
    //void folderViewCallback();
    void folderNewCallback();
    void folderImportCallback();
    void folderDeleteCallback();
    void folderRenameCallback();
    
    void folderNewUpdate(Gtk_Entry* entry);
    void folderImportUpdate(Gtk_FileSelection* fileSel);
    void folderDeleteUpdate();
    void folderRenameUpdate(Gtk_Entry* entry);
    
    void optionsChangeAddressCallback();
    
    void optionsChangeAddressUpdate(Gtk_Entry* entry);
    
    void messageViewCallback();
    void messageReplyCallback();
    void messageComposeCallback();
    void messageDeleteCallback();
    void messageMoveToCallback(string menuItem);
    void resetList();  

    void folderComboActive();
    
    Gtk_Menu* messageMenu;  
    Gtk_Menu* messageMoveToMenu;  
    Gtk_MenuItem* messageMoveToItem;  
    
    Gtk_VBox* vbox;
    Gtk_HBox* hbox;
    
    Gtk_ScrolledWindow* folderListWin;
    Gtk_CList* folderList;
    Gtk_ScrolledWindow* messageListWin;
    Gtk_CList* messageList;
    
    Gtk_AccelGroup* accelGroup;
    Gtk_ItemFactory_MenuBar* itemFactory;
    gint folderSelect;
    gint messageSelect;
    int numRows;
    int numFolder;

    Gtk_HBox* folderHBox;
    Gtk_Label* folderLabel;
    Gtk_Combo* folderCombo;
    GList* folderGList;
};

#endif
