/*
 * GtkWriteWin.C - source file for class GtkWriteWin
 * Copyright (c) 1999 Joe Yandle <yandle@cs.unc.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "Model.h"
#include "GtkWriteWin.h"

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

const int BSIZE=1024;

/*
  GtkWriteWin::GtkWriteWin() - 
*/
GtkWriteWin::GtkWriteWin(Model* m) : GtkMessageWin(m)
{
    numAttach=0;

    attachCombo = new Gtk_Combo();
    attachList=NULL;
    
    //char buffer[BSIZE];
    
    itemFactory->create_item("/Message/Send", "<control>S", "<Item>", ItemFactoryConnector<GtkWriteWin,string>(this,&messageCallback,"Send"));
    itemFactory->create_item("/Attach", 0, "<Branch>", 0);
    itemFactory->create_item("/Attach/Attach File", "<control>T", "<Item>", ItemFactoryConnector<GtkWriteWin,string>(this,&attachCallback,"Attach File"));
    addHelpMenu();
    
    subjLabel = new Gtk_Label("Subject: ");
    subjLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    toLabel = new Gtk_Label("To: ");
    toLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    attachLabel = new Gtk_Label("Attach: ");
    attachLabel->set_justify(GTK_JUSTIFY_LEFT);  
    
    subjEntry = new Gtk_Entry();
    toEntry = new Gtk_Entry();
    
    
    //Gtk_VBox* winVBox = new Gtk_VBox(FALSE, 0);
    
    headerHBox = new Gtk_HBox(FALSE, 0);
    subjLabelHBox = new Gtk_HBox(FALSE, 0);
    subjEntryHBox = new Gtk_HBox(FALSE, 0);
    toLabelHBox = new Gtk_HBox(FALSE, 0);
    toEntryHBox = new Gtk_HBox(FALSE, 0);
    attachLabelHBox = new Gtk_HBox(FALSE, 0);
    attachComboHBox = new Gtk_HBox(FALSE, 0);
    
    subjLabelHBox->pack_start(*subjLabel, FALSE, FALSE, 0);
    toLabelHBox->pack_start(*toLabel, FALSE, FALSE, 0);
    attachLabelHBox->pack_start(*attachLabel, FALSE, FALSE, 0);
    
    subjEntryHBox->pack_start(*subjEntry, TRUE, TRUE, 0);
    toEntryHBox->pack_start(*toEntry, TRUE, TRUE, 0);
    attachComboHBox->pack_start(*attachCombo, TRUE, TRUE, 0);
    
/*
  subjToLabelVBox->pack_start(*subjLabelHBox, FALSE, FALSE, 0);
  subjToLabelVBox->pack_start(*toLabelHBox, FALSE, FALSE, 0);
  
  subjToEntryVBox->pack_start(*subjEntryHBox, FALSE, FALSE, 0);
  subjToEntryVBox->pack_start(*toEntryHBox, FALSE, FALSE, 0);
  
*/
    
    headerTable = new Gtk_Table(2,2,FALSE);
    
    headerTable->attach(*subjLabelHBox, 0,1,1,2,0,0,0,0);
    headerTable->attach(*toLabelHBox, 0,1,0,1,0,0,0,0);
    headerTable->attach(*subjEntryHBox, 1,2,1,2);
    headerTable->attach(*toEntryHBox, 1,2,0,1);
    headerTable->attach(*attachLabelHBox, 0, 1, 4, 5, 0, 0, 0, 0); 
    headerTable->attach(*attachComboHBox, 1, 2, 4, 5); 
    
    headerHBox->pack_start(*headerTable, TRUE, TRUE, 10);
    
    int* please = new int;
    *please=0;
    
    messageText->set_editable(TRUE);
    
    char sigFilePath[BSIZE];
    char sigBuffer[BSIZE];

    sprintf(sigFilePath, "%s/.signature", getenv("HOME"));
    FILE* sigFile = fopen(sigFilePath, "r");

    int textPos = 0;
    int bytesRead;
    if(sigFile) {
	messageText->insert_text("\n\n", 2 , &textPos);
	while(!feof(sigFile)) {
	    bytesRead = fread(sigBuffer, sizeof(char), BSIZE, sigFile);
	    messageText->insert_text(sigBuffer, bytesRead, &textPos);
	}
    }
    
    messageWin->add(messageText);
    
    vbox->pack_start(*headerHBox, FALSE, FALSE, 10);
    vbox->pack_start(*messageWin, TRUE, TRUE, 0);
    
    show_all();
    
}

/*
  GtkWriteWin::~GtkWriteWin()
*/
GtkWriteWin::~GtkWriteWin()
{
    //printf("Calling destructor\n");
    subjLabel->hide();
    delete subjLabel;
    
    subjEntry->hide();
    delete subjEntry;

    toLabel->hide();
    delete toLabel;
    
    toEntry->hide();
    delete toEntry;

    //g_list_foreach(attachList, deleteNodes, NULL);
    for(int i=0;i<numAttach;i++) {
	delete g_list_nth(attachList,i)->data;
    }
    g_list_free(attachList);

    attachCombo->hide();
    delete attachCombo;

    messageText->hide();
    delete messageText;
    messageWin->hide();
    delete messageWin;
    
    delete accelGroup;
    delete itemFactory;
    
    headerTable->hide();
    delete headerTable;
    headerHBox->hide();
    delete headerHBox;
    
    subjLabelHBox->hide();
    delete subjLabelHBox;
    subjEntryHBox->hide();
    delete subjEntryHBox;
    toLabelHBox->hide();
    delete toLabelHBox;
    toEntryHBox->hide();
    delete toEntryHBox;
    attachLabelHBox->hide();
    delete attachLabelHBox;
    attachComboHBox->hide();
    delete attachComboHBox;    

    vbox->hide();
    delete vbox;
    hbox->hide();
    delete hbox;
}



/*
  void escapeQuote(char* dst, char* src);
*/
void GtkWriteWin::escapeQuote(char* dst, char* src)
{
    int j=0;
    for(unsigned int i=0;i<strlen(src);i++) {
	if(src[i]=='"') {
	    dst[j++]='\\';
	    
	}
	dst[j++]=src[i];
    }
    dst[j]=0;
}

/*
  void messageSendCallback();
*/
void GtkWriteWin::messageSendCallback()
{
    _gtk_string subjTextString, toTextString, messageString;
    char *subjText, *toText, *messageTxt, *fromText, *sendText, *contentText;
    
    char* toHead = "To: ";
    char* fromHead = "From: ";
    char* subjHead = "Subject: ";
    char* mailHead = "X-Mailer: gtkmail-0.1.1";
    char* mimeHead = "Mime-Version: 1.0";
    char* contentHead = "Content-Type: ";
    char* newLine = "\n";
    char* textPlain = "text/plain; charset=iso-8859-1";
    char* multiMixed = "multipart/mixed";


    subjTextString = subjEntry->get_text();
    toTextString = toEntry->get_text();
    messageString = messageText->get_chars(0, messageText->get_length());
    
    subjText = new char[2*strlen(subjTextString.c_str())+1];
    toText = new char[2*strlen(toTextString.c_str())+1];
    messageTxt = new char[2*strlen(messageString.c_str())+1];
    fromText = new char[2*strlen(model->getUserAddress())+1];
    sendText = new char[2*strlen(model->getUserAddress())+1];
    
/*
  escapeQuote(toText, (char*)toTextString.c_str());
  escapeQuote(subjText, (char*)subjTextString.c_str());
  escapeQuote(messageTxt, (char*)messageString.c_str());
  escapeQuote(fromText, model->getUserAddress());
*/
    
    strcpy(toText, (char*)toTextString.c_str());
    strcpy(subjText, (char*)subjTextString.c_str());
    strcpy(messageTxt, (char*)messageString.c_str());
    strcpy(fromText, model->getUserAddress());
    escapeQuote(sendText, model->getUserAddress());
    //strcpy(sendText, model->getUserAddress());
    
    if(strlen(toText) == 0) {
	return;
    }
    
    char* command = new char[strlen(toText) + 1024];
    //sprintf(command, "sendmail \"%s\"", toText);
    sprintf(command, "sendmail -t -f \"%s\"", sendText);


    int spit[2];
    if(pipe(spit) == -1) {
	perror("Error opening pipe");
	exit(1);
    }
    if(!fork()) {
	dup2(spit[0], 0);
	close(spit[1]);
	execl(SENDMAIL, "sendmail", "-t", "-f", sendText, NULL);
    }
    else {
	if(numAttach == 0) {
	    contentText = new char[strlen(textPlain)+1];
	    strcpy(contentText, textPlain);
	}
	else {
	    contentText = new char[BSIZE];
	    char bound[32];
	    for(int i=0;i<16;i++) {
		bound[i]=(rand() % 26) + 65;
	    }
	    bound[16]=0;
	    sprintf(contentText, "%s; %s=\"%s\"", multiMixed, "boundary", bound);
	    
	    int sizeGuess = strlen(messageTxt);
	    
	    struct stat mystat;
	    
	    for(int j=0; j<numAttach; j++) {
		stat((char*)(g_list_nth(attachList, j)->data), &mystat);
		sizeGuess += 2*mystat.st_size;
	    }
	    //printf("Size guess=%d\n", sizeGuess);

	    char* tempMessageTxt = new char[sizeGuess + BSIZE];
	    tempMessageTxt[0]=0;
	    char attachType[BSIZE], attachCode[BSIZE], attachHead[BSIZE];

	    sprintf(attachType, "Content-Type: %s", textPlain);
	    sprintf(attachCode, "Content-Transfer-Encoding: %s", "7bit");
	    sprintf(attachHead, "--%s\n%s\n%s\n\n", bound, attachType, attachCode);
	    strcat(tempMessageTxt, attachHead);
	    strcat(tempMessageTxt, messageTxt);
	    strcat(tempMessageTxt, newLine);
	    strcat(tempMessageTxt, newLine);

	    for(int j=0; j<numAttach; j++) {
		char attachPath[BSIZE];
		strcpy(attachPath, (char*)(g_list_nth(attachList, j)->data));
		stat(attachPath, &mystat);
		//printf("%s\n",attachPath);		

		sprintf(attachType, "Content-Type: %s; name=\"%s\"", "application/octet-stream", attachPath);
		sprintf(attachCode, "Content-Transfer-Encoding: %s", "base64");
		sprintf(attachHead, "--%s\n%s\n%s\n\n", bound, attachType, attachCode);
		strcat(tempMessageTxt, attachHead);

		int swallow[2];
		if(pipe(swallow) == -1) {
		    perror("Error opening pipe");
		    exit(1);
		}
		
		if(!fork()) {
		    dup2(swallow[1], 1);
		    close(swallow[0]);
		    execl(MIMENCODE, "mimencode", attachPath, NULL);
		}
		char* temp2 = new char[2*mystat.st_size];

		close(swallow[1]);

		int bytesRead = -1;

		while(bytesRead != 0) {
		    bytesRead = read(swallow[0], temp2, 2*mystat.st_size - 1);
		    temp2[bytesRead]=0;
		    strcat(tempMessageTxt, temp2);
		}

		strcat(tempMessageTxt, newLine);
		strcat(tempMessageTxt, newLine);
		delete temp2;
		close(swallow[0]);
	    }

	    sprintf(attachHead, "---%s\n\n", bound);
	    strcat(tempMessageTxt, attachHead);

	    delete messageTxt;
	    messageTxt = tempMessageTxt;
		    
	}
	char tempHeaders[16*BSIZE];
	sprintf(tempHeaders, "%s%s\n%s%s\n%s%s\n%s\n%s%s\n%s\n\n", 
		toHead, toText, fromHead, fromText, subjHead, subjText, mimeHead, contentHead, contentText, mailHead);

	close(spit[0]);
	write(spit[1], tempHeaders, strlen(tempHeaders));
	write(spit[1], messageTxt, strlen(messageTxt));
	write(spit[1], newLine, strlen(newLine));
	close(spit[1]);

	delete [] command;
	delete [] toText;
	delete [] fromText;
	delete [] subjText;
	delete [] messageTxt;
	delete [] contentText;
	delete_self();
    }
}

/*
  void setTo(char* to)
*/
void GtkWriteWin::setTo(char* to)
{
    toEntry->set_text(to);
}

/*
  void setSubject(char* to);
*/
void GtkWriteWin::setSubject(char* to)
{
    subjEntry->set_text(to);
}

/*
  void GtkWriteWin::messageCallback(string s)
*/
void GtkWriteWin::messageCallback(string s)
{
    if(s == "Send") {
	messageSendCallback();
    }
}

/*
  void GtkReadWin::attachCallback(string s)
*/
void GtkWriteWin::attachCallback(string s)
{
    if(s == "Attach File") {
	attachFileCallback();
    }
}

/*
  void attachFileCallback(Gtk_FileSelection* fileSel);
*/
void GtkWriteWin::attachFileCallback()
{
    Gtk_FileSelection* fileSel = new Gtk_FileSelection("Attach File");
    fileSel->set_position(GTK_WIN_POS_CENTER);
    connect_to_method(fileSel->get_ok_button()->clicked, this, &attachFileUpdate, fileSel);
    connect_to_method(fileSel->get_cancel_button()->clicked, (GtkMessageWin*)this, &GtkMessageWin::killWidget, (Gtk_Widget*)fileSel);
    connect_to_method(fileSel->get_ok_button()->clicked, (GtkMessageWin*)this, &GtkMessageWin::killWidget, (Gtk_Widget*)fileSel);
    
    fileSel->show_all();    
}

/*
  void attachFileUpdate(Gtk_FileSelection* fileSel);
*/
void GtkWriteWin::attachFileUpdate(Gtk_FileSelection* fileSel)
{
    char* fileName = new char[strlen((char*)(fileSel->get_filename().c_str()))+1];
    strcpy(fileName, (char*)(fileSel->get_filename().c_str()));
    attachList = g_list_append(attachList, fileName);
    attachCombo->set_popdown_strings(attachList);
    numAttach++;
}

