/*-*- c++ -*-********************************************************
 * FolderView.h : a window which shows a MailFolder                 *
 *                                                                  *
 * (C) 1997 by Karsten Ballder (Ballueder@usa.net)                 *
 *                                                                  *
 * $Id: FolderView.h,v 1.9 1999/05/12 16:51:13 VZ Exp $
 *
 *******************************************************************/

#ifndef FOLDERVIEW_H
#define FOLDERVIEW_H

#include "MEvent.h"

class MailFolder;
class ProfileBase;

/**
   FolderView class, a window displaying a MailFolder.
*/
class FolderView : public MEventReceiver
{
public:
   FolderView()
      {
         m_regCookieTreeChange = MEventManager::Register(*this, MEventId_FolderTreeChange);
         ASSERT_MSG( m_regCookieTreeChange, "can't reg folder view with event manager");
         m_regCookieFolderUpdate = MEventManager::Register(*this, MEventId_FolderUpdate);
         ASSERT_MSG( m_regCookieFolderUpdate, "can't reg folder view with event manager");
      }
   /// update the user interface
   virtual void Update(void) = 0;
   /// virtual destructor
   virtual ~FolderView()
      {
         MEventManager::Deregister(m_regCookieTreeChange);
         MEventManager::Deregister(m_regCookieFolderUpdate);
      }

   /// event processing function
   virtual bool OnMEvent(MEventData& ev)
   {
      if ( ev.GetId() == MEventId_FolderTreeChange )
      {
         MEventFolderTreeChangeData& event = (MEventFolderTreeChangeData &)ev;
         if ( event.GetChangeKind() == MEventFolderTreeChangeData::Delete )
            OnFolderDeleteEvent(event.GetFolderFullName());
      }
      else if ( ev.GetId() == MEventId_FolderUpdate)
         OnFolderUpdateEvent((MEventFolderUpdateData&)ev);
  
      return true; // continue evaluating this event
   }

   /// return full folder name
   const String& GetFullName() { return m_folderName; }

   /// return a profile pointer:
   ProfileBase *GetProfile(void) const { return m_Profile; }

   /// return pointer to folder
   MailFolder * GetFolder(void) const { return m_MailFolder; }

protected:
   /// the derived class should close when our folder is deleted
   virtual void OnFolderDeleteEvent(const String& folderName) = 0;
   /// the derived class should update their display
   virtual void OnFolderUpdateEvent(MEventFolderUpdateData &event) = 0;

   /// the profile we use for our settings
   ProfileBase *m_Profile;
   /// full folder name of the folder we show
   String m_folderName;
   /// the mail folder being displayed
   MailFolder *m_MailFolder;

private:
   void *m_regCookieTreeChange, *m_regCookieFolderUpdate;
};
#endif
